@@ -1,7 +1,7 @@
 # Copyright (c) 2004 CentralNic Ltd. All rights reserved. This program is
 # free software; you can redistribute it and/or modify it under the same
 # terms as Perl itself.
-# $Id: CNic.pm,v 1.31 2005/04/20 12:46:40 gavin Exp $
+# $Id: CNic.pm,v 1.37 2006/01/09 10:44:54 gavin Exp $
 package WWW::CNic;
 use LWP;
 use LWP::ConnCache;
@@ -11,7 +11,7 @@ use Digest::MD5 qw(md5_hex);
 use vars qw($VERSION $CONNECTION_CACHE);
 use strict;
 
-our $VERSION = '0.13';
+our $VERSION = '0.16';
 
 =pod
 
@@ -193,7 +193,7 @@ sub set {
 	my $self = shift;
 	my %params = @_;
 	foreach my $name(keys %params) {
-		$self->{_params}{$name} = $params{$name};
+		$self->{_params}->{$name} = $params{$name};
 	}
 	return;
 }
@@ -205,10 +205,12 @@ sub execute {
 		$self->{_command} eq 'search'		&& return $self->_search();
 		$self->{_command} eq 'suffixes'		&& return $self->_suffixes();
 		$self->{_command} eq 'create_handle'	&& return $self->_create_handle();
+		$self->{_command} eq 'handle_info'	&& return $self->_handle_info();
 		$self->{_command} eq 'register'		&& return $self->_register();
 		$self->{_command} eq 'register_idn'	&& return $self->_register(idn => 1);
 		$self->{_command} eq 'register_atgb'	&& return $self->_register_atgb();
 		$self->{_command} eq 'modify'		&& return $self->_modify();
+		$self->{_command} eq 'modify_handle'	&& return $self->_modify_handle();
 		$self->{_command} eq 'renewals'		&& return $self->_upcoming_renewals();
 		$self->{_command} eq 'list_domains'	&& return $self->_list_domains();
 		$self->{_command} eq 'issue_renewals'	&& return $self->_issue_renewals();
@@ -219,6 +221,7 @@ sub execute {
 		$self->{_command} eq 'start_transfer'	&& return $self->_start_transfer();
 		$self->{_command} eq 'check_transfer'	&& return $self->_check_transfer();
 		$self->{_command} eq 'cancel_transfer'	&& return $self->_cancel_transfer();
+		$self->{_command} eq 'reactivate'	&& return $self->_reactivate_domain();
 		die("Invalid command '$self->{_command}'");
 	}
 }
@@ -226,39 +229,40 @@ sub execute {
 sub _whois {
 	my $self = shift;
 	die("Missing domain name") if $self->{_domain} eq '';
-	$self->{_response}{_raw} = $self->_get(GET("$self->{_base}/wwwhois?domain=$self->{_domain}&test=$self->{_test}"));
+	$self->{_response}->{_raw} = $self->_get(GET("$self->{_base}/wwwhois?domain=$self->{_domain}&test=$self->{_test}"));
 	use WWW::CNic::Response::Whois;
-	return WWW::CNic::Response::Whois->new($self->{_response}{_raw});
+	return WWW::CNic::Response::Whois->new($self->{_response}->{_raw});
 }
 
 sub _search {
 	my $self = shift;
 	die("Missing domain name") if $self->{_domain} eq '';
 	my $url = "$self->{_base}/search?domain=$self->{_domain}";
-	if (defined(@{$self->{_params}{suffixlist}})) {
-		$url .= '&suffixlist='.join(',', @{$self->{_params}{suffixlist}}).'&test='.$self->{_test};
+	if (defined(@{$self->{_params}->{suffixlist}})) {
+		$url .= '&suffixlist='.join(',', @{$self->{_params}->{suffixlist}}).'&test='.$self->{_test};
 	}
-	$self->{_response}{_raw} = $self->_get(GET($url));
+	$self->{_response}->{_raw} = $self->_get(GET($url));
 	use WWW::CNic::Response::Search;
-	return WWW::CNic::Response::Search->new($self->{_response}{_raw});
+	return WWW::CNic::Response::Search->new($self->{_response}->{_raw});
 }
 
 sub _suffixes {
 	my $self = shift;
-	$self->{_response}{_raw} = $self->_get(GET("$self->{_base}/suffixes?test=$self->{_test}"));
+	$self->{_response}->{_raw} = $self->_get(GET("$self->{_base}/suffixes?test=$self->{_test}"));
 	use WWW::CNic::Response::Suffixes;
-	return WWW::CNic::Response::Suffixes->new($self->{_response}{_raw});
+	return WWW::CNic::Response::Suffixes->new($self->{_response}->{_raw});
 }
 
 sub _create_handle {
 	my $self = shift;
-	die("Invalid type for handle") unless (ref($self->{_params}{handle}) eq 'HASH');
-	my $handle = $self->_build_handle_string($self->{_params}{handle});
+	die("Invalid type for handle") unless (ref($self->{_params}->{handle}) eq 'HASH');
+	my $handle = $self->_build_handle_string($self->{_params}->{handle});
 	my %params =  (
 		user		=> $self->{_username},
 		password	=> $self->_crypt_md5($self->{_password}),
 		test		=> $self->{_test},
 		handle		=> $handle,
+		visible		=> ($self->{_params}->{visible} == 0 ? 0 : 1),
 	);
 	my $req = HTTP::Request->new(POST => "$self->{_base}/create_handle");
 	$req->content_type('application/x-www-form-urlencoded');
@@ -273,9 +277,23 @@ sub _create_handle {
 		push(@content, $pair);
 	}
 	$req->content(join('&', @content));
-	$self->{_response}{_raw} = $self->_get($req);
+	$self->{_response}->{_raw} = $self->_get($req);
 	use WWW::CNic::Response::CreateHandle;
-	return WWW::CNic::Response::CreateHandle->new($self->{_response}{_raw});
+	return WWW::CNic::Response::CreateHandle->new($self->{_response}->{_raw});
+}
+
+sub _handle_info {
+	my $self = shift;
+	my %params =  (
+		user		=> $self->{_username},
+		password	=> $self->_crypt_md5($self->{_password}),
+		test		=> $self->{_test},
+		handle		=> $self->{_params}->{handle},
+	);
+	my $req = POST("$self->{_base}/handle_info", \%params);
+	$self->{_response}->{_raw} = $self->_get($req);
+	use WWW::CNic::Response::HandleInfo;
+	return WWW::CNic::Response::HandleInfo->new($self->{_response}->{_raw});
 }
 
 sub _register {
@@ -285,16 +303,16 @@ sub _register {
 	die("Missing username") if $self->{_username} eq '';
 	die("Missing password") if $self->{_password} eq '';
 	my ($domain, $suffix) = split(/\./, $self->{_domain}, 2);
-	my $chandle = (ref($self->{_params}{chandle}) eq 'HASH' ? $self->_build_handle_string($self->{_params}{chandle}) : $self->{_params}{chandle});
-	my $thandle = (ref($self->{_params}{thandle}) eq 'HASH' ? $self->_build_handle_string($self->{_params}{thandle}) : $self->{_params}{thandle});
+	my $chandle = (ref($self->{_params}->{chandle}) eq 'HASH' ? $self->_build_handle_string($self->{_params}->{chandle}) : $self->{_params}->{chandle});
+	my $thandle = (ref($self->{_params}->{thandle}) eq 'HASH' ? $self->_build_handle_string($self->{_params}->{thandle}) : $self->{_params}->{thandle});
 	my $dns;
 	my $dns_param = 'dns[]';
-	if (ref($self->{_params}{dns}) eq 'ARRAY') {
-		$dns = join('&dns[]=', @{$self->{_params}{dns}});
-	} elsif (ref($self->{_params}{dns}) eq 'HASH') {
+	if (ref($self->{_params}->{dns}) eq 'ARRAY') {
+		$dns = join('&dns[]=', @{$self->{_params}->{dns}});
+	} elsif (ref($self->{_params}->{dns}) eq 'HASH') {
 		my @entries;
-		foreach my $name(keys %{$self->{_params}{dns}}) {
-			push(@entries, $name.'::'.${$self->{_params}{dns}}{$name});
+		foreach my $name(keys %{$self->{_params}->{dns}}) {
+			push(@entries, $name.'::'.${$self->{_params}->{dns}}->{$name});
 		}
 		$dns = join('&dns[]=', @entries);
 	} elsif ($dns == 'defaults') {
@@ -307,10 +325,10 @@ sub _register {
 				test		=> $self->{_test},
 				domain		=> $domain,
 				suffix		=> $suffix,
-				registrant	=> $self->{_params}{registrant},
+				registrant	=> $self->{_params}->{registrant},
 				chandle		=> $chandle,
 				thandle		=> $thandle,
-				period		=> $self->{_params}{period},
+				period		=> $self->{_params}->{period},
 				$dns_param	=> $dns,
 			);
 	my $POST_URL = $self->{_base}.'/'.($args{idn} == 1 ? 'register_idn' : 'register');
@@ -327,9 +345,9 @@ sub _register {
 		push(@content, $pair);
 	}
 	$req->content(join('&', @content));
-	$self->{_response}{_raw} = $self->_get($req);
+	$self->{_response}->{_raw} = $self->_get($req);
 	use WWW::CNic::Response::Register;
-	return WWW::CNic::Response::Register->new($self->{_response}{_raw});
+	return WWW::CNic::Response::Register->new($self->{_response}->{_raw});
 }
 
 sub _register_atgb {
@@ -345,10 +363,10 @@ sub _register_atgb {
 				fname		=> $fname,
 				sname		=> $sname,
 				suffix		=> $suffix,
-				registrant	=> $self->{_params}{registrant},
-				handle		=> $self->_build_handle_string($self->{_params}{handle}),
-				user_password	=> $self->{_params}{user_password},
-				send_email	=> $self->{_params}{send_email}
+				registrant	=> $self->{_params}->{registrant},
+				handle		=> $self->_build_handle_string($self->{_params}->{handle}),
+				user_password	=> $self->{_params}->{user_password},
+				send_email	=> $self->{_params}->{send_email}
 			);
 	my $req = HTTP::Request->new(POST => "$self->{_base}/register_atgb");
 	$req->content_type('application/x-www-form-urlencoded');
@@ -358,9 +376,9 @@ sub _register_atgb {
 		push(@content, $pair);
 	}
 	$req->content(join('&', @content));
-	$self->{_response}{_raw} = $self->_get($req);
+	$self->{_response}->{_raw} = $self->_get($req);
 	use WWW::CNic::Response::Register::AtGB;
-	return WWW::CNic::Response::Register::AtGB->new($self->{_response}{_raw});
+	return WWW::CNic::Response::Register::AtGB->new($self->{_response}->{_raw});
 }
 
 sub _modify {
@@ -374,18 +392,26 @@ sub _modify {
 				test		=> $self->{_test},
 				domain		=> $self->{_domain},
 			);
-	if (defined($self->{_params}{thandle})) {
-		$params{thandle} = (ref($self->{_params}{thandle}) eq 'HASH' ? $self->_build_handle_string($self->{_params}{thandle}) : $self->{_params}{thandle});
+	if (defined($self->{_params}->{chandle})) {
+		$params{chandle} = (ref($self->{_params}->{chandle}) eq 'HASH' ? $self->_build_handle_string($self->{_params}->{chandle}) : $self->{_params}->{chandle});
 	}
-	if (defined($self->{_params}{dns})) {
+	if (defined($self->{_params}->{thandle})) {
+		$params{thandle} = (ref($self->{_params}->{thandle}) eq 'HASH' ? $self->_build_handle_string($self->{_params}->{thandle}) : $self->{_params}->{thandle});
+	}
+	$params{ttl} = int($self->{_params}->{ttl}) if (defined($self->{_params}->{ttl}));
+
+	if (defined($self->{_params}->{dns})) {
 		my @dns;
-		if (defined(@{$self->{_params}{dns}{drop}})) {
-			foreach my $name(@{$self->{_params}{dns}{drop}}) {
+		if ($self->{_params}->{dns}->{drop} eq 'all') {
+			push(@dns, "drop:all");
+
+		} elsif (defined(@{$self->{_params}->{dns}->{drop}})) {
+			foreach my $name(@{$self->{_params}->{dns}->{drop}}) {
 				push(@dns, "drop:$name");
 			}
 		}
-		if (defined(@{$self->{_params}{dns}{add}})) {
-			foreach my $name(@{$self->{_params}{dns}{add}}) {
+		if (defined(@{$self->{_params}->{dns}->{add}})) {
+			foreach my $name(@{$self->{_params}->{dns}->{add}}) {
 				push(@dns, "add:$name");
 			}
 		}
@@ -404,9 +430,38 @@ sub _modify {
 		push(@content, $pair);
 	}
 	$req->content(join('&', @content));
-	$self->{_response}{_raw} = $self->_get($req);
+	$self->{_response}->{_raw} = $self->_get($req);
 	use WWW::CNic::Response::Modify;
-	return WWW::CNic::Response::Modify->new($self->{_response}{_raw});
+	return WWW::CNic::Response::Modify->new($self->{_response}->{_raw});
+}
+
+sub _modify_handle {
+	my $self = shift;
+	die("Missing username") if $self->{_username} eq '';
+	die("Missing password") if $self->{_password} eq '';
+	die("Missing handle ID") if ($self->{_params}->{handle} eq '');
+
+	my %params = (
+		user		=> $self->{_username},
+		password	=> $self->_crypt_md5($self->{_password}),
+		test		=> $self->{_test},
+	);
+	foreach my $key (keys(%{$self->{_params}})) {
+		$params{$key} = $self->{_params}->{$key};
+	}
+
+	my $req = HTTP::Request->new( POST => "$self->{_base}/modify_handle");
+	$req->content_type('application/x-www-form-urlencoded');
+	my @content;
+
+	foreach my $name(keys %params) {
+		push(@content, uri_escape($name).'='.uri_escape($params{$name}));
+	}
+	$req->content(join('&', @content));
+
+	$self->{_response}->{_raw} = $self->_get($req);
+	use WWW::CNic::Response::ModifyHandle;
+	return WWW::CNic::Response::ModifyHandle->new($self->{_response}->{_raw});
 }
 
 sub _upcoming_renewals {
@@ -415,9 +470,9 @@ sub _upcoming_renewals {
 	die("Missing password") if $self->{_password} eq '';
 	my $url = "$self->{_base}/upcoming_renewals?user=$self->{_username}&months=$self->{_months}&password=".$self->_crypt_md5($self->{_password}).'&test='.$self->{_test};
 	my $req = HTTP::Request->new(GET => $url);
-	$self->{_response}{_raw} = $self->_get($req);
+	$self->{_response}->{_raw} = $self->_get($req);
 	use WWW::CNic::Response::Renewals;
-	return WWW::CNic::Response::Renewals->new($self->{_response}{_raw});
+	return WWW::CNic::Response::Renewals->new($self->{_response}->{_raw});
 }
 
 sub _list_domains {
@@ -428,21 +483,21 @@ sub _list_domains {
 				user		=> $self->{_username},
 				password	=> $self->_crypt_md5($self->{_password}),
 				test		=> $self->{_test},
-				offset		=> $self->{_params}{offset},
-				length		=> $self->{_params}{length},
-				orderby		=> $self->{_params}{orderby},
+				offset		=> $self->{_params}->{offset},
+				length		=> $self->{_params}->{length},
+				orderby		=> $self->{_params}->{orderby},
 			);
 	my $req = POST("$self->{_base}/list_domains", \%params);
-	$self->{_response}{_raw} = $self->_get($req);
+	$self->{_response}->{_raw} = $self->_get($req);
 	use WWW::CNic::Response::DomainList;
-	return WWW::CNic::Response::DomainList->new($self->{_response}{_raw});
+	return WWW::CNic::Response::DomainList->new($self->{_response}->{_raw});
 }
 
 sub _issue_renewals {
 	my $self = shift;
 	die("Missing username") if $self->{_username} eq '';
 	die("Missing password") if $self->{_password} eq '';
-	die("Missing domains") if scalar(@{$self->{_params}{domains}}) < 1;
+	die("Missing domains") if scalar(@{$self->{_params}->{domains}}) < 1;
 	my %params = 	(
 				user		=> $self->{_username},
 				password	=> $self->_crypt_md5($self->{_password}),
@@ -451,14 +506,14 @@ sub _issue_renewals {
 				immediate	=> $self->{_params}->{immediate},
 			);
 	my $i = 0;
-	foreach my $domain(@{$self->{_params}{domains}}) {
+	foreach my $domain(@{$self->{_params}->{domains}}) {
 		$params{"domains[$i]"} = $domain;
 		$i++;
 	}
 	my $req = POST("$self->{_base}/issue_renewals", \%params);
-	$self->{_response}{_raw} = $self->_get($req);
+	$self->{_response}->{_raw} = $self->_get($req);
 	use WWW::CNic::Response::IssueRenewals;
-	return WWW::CNic::Response::IssueRenewals->new($self->{_response}{_raw});
+	return WWW::CNic::Response::IssueRenewals->new($self->{_response}->{_raw});
 }
 
 sub _get_pricing {
@@ -473,29 +528,30 @@ sub _get_pricing {
 	);
 	my $url = "$self->{_base}/get_pricing";
 	my $req = POST($url, \%params);
-	$self->{_response}{_raw} = $self->_get($req);
+	$self->{_response}->{_raw} = $self->_get($req);
 	use WWW::CNic::Response::GetPricing;
-	return WWW::CNic::Response::GetPricing->new($self->{_response}{_raw});
+	return WWW::CNic::Response::GetPricing->new($self->{_response}->{_raw});
 }
 
 sub _delete_domain {
 	my $self = shift;
 	die("Missing username") if $self->{_username} eq '';
 	die("Missing password") if $self->{_password} eq '';
-	die("Missing reason")   if $self->{_reason}   eq '';
 	die("Missing domain")	if $self->{_domain}   eq '';
 	my %params = (
 				user		=> $self->{_username},
 				password	=> $self->_crypt_md5($self->{_password}),
 				test		=> $self->{_test},
 				domain		=> $self->{_domain},
-				reason		=> $self->{_reason},
+				reason		=> $self->{_reason} || $self->{_params}->{reason},
 	);
+	die("Missing reason") if ($params{reason}  eq '');
+	$params{immediate} = 1 if ($self->{_params}->{immediate});
 	my $url = "$self->{_base}/delete_domain";
 	my $req = POST($url, \%params);
-	$self->{_response}{_raw} = $self->_get($req);
+	$self->{_response}->{_raw} = $self->_get($req);
 	use WWW::CNic::Response::DeleteDomain;
-	return WWW::CNic::Response::DeleteDomain->new($self->{_response}{_raw});
+	return WWW::CNic::Response::DeleteDomain->new($self->{_response}->{_raw});
 }
 
 sub _decline_domain {
@@ -511,9 +567,9 @@ sub _decline_domain {
 	);
 	my $url = "$self->{_base}/decline_domain";
 	my $req = POST($url, \%params);
-	$self->{_response}{_raw} = $self->_get($req);
+	$self->{_response}->{_raw} = $self->_get($req);
 	use WWW::CNic::Response::DeclineDomain;
-	return WWW::CNic::Response::DeclineDomain->new($self->{_response}{_raw});
+	return WWW::CNic::Response::DeclineDomain->new($self->{_response}->{_raw});
 }
 
 sub _undecline_domain {
@@ -529,31 +585,31 @@ sub _undecline_domain {
 	);
 	my $url = "$self->{_base}/undecline_domain";
 	my $req = POST($url, \%params);
-	$self->{_response}{_raw} = $self->_get($req);
+	$self->{_response}->{_raw} = $self->_get($req);
 	use WWW::CNic::Response::UnDeclineDomain;
-	return WWW::CNic::Response::UnDeclineDomain->new($self->{_response}{_raw});
+	return WWW::CNic::Response::UnDeclineDomain->new($self->{_response}->{_raw});
 }
 
 sub _start_transfer {
 	my $self = shift;
 	die("Missing username") if $self->{_username} eq '';
 	die("Missing password") if $self->{_password} eq '';
-	die("Missing domains") if scalar(@{$self->{_params}{domains}}) < 1;
+	die("Missing domains") if scalar(@{$self->{_params}->{domains}}) < 1;
 	my %params = (
 				user		=> $self->{_username},
 				password	=> $self->_crypt_md5($self->{_password}),
 				test		=> $self->{_test},
 	);
 	my $i = 0;
-	foreach my $domain(@{$self->{_params}{domains}}) {
+	foreach my $domain(@{$self->{_params}->{domains}}) {
 		$params{"domains[$i]"} = $domain;
 		$i++;
 	}
 	my $url = "$self->{_base}/start_transfer";
 	my $req = POST($url, \%params);
-	$self->{_response}{_raw} = $self->_get($req);
+	$self->{_response}->{_raw} = $self->_get($req);
 	use WWW::CNic::Response::StartTransfer;
-	return WWW::CNic::Response::StartTransfer->new($self->{_response}{_raw});
+	return WWW::CNic::Response::StartTransfer->new($self->{_response}->{_raw});
 }
 
 sub _check_transfer {
@@ -569,9 +625,9 @@ sub _check_transfer {
 	);
 	my $url = "$self->{_base}/check_transfer";
 	my $req = POST($url, \%params);
-	$self->{_response}{_raw} = $self->_get($req);
+	$self->{_response}->{_raw} = $self->_get($req);
 	use WWW::CNic::Response::CheckTransfer;
-	return WWW::CNic::Response::CheckTransfer->new($self->{_response}{_raw});
+	return WWW::CNic::Response::CheckTransfer->new($self->{_response}->{_raw});
 }
 
 sub _cancel_transfer {
@@ -587,9 +643,28 @@ sub _cancel_transfer {
 	);
 	my $url = "$self->{_base}/cancel_transfer";
 	my $req = POST($url, \%params);
-	$self->{_response}{_raw} = $self->_get($req);
+	$self->{_response}->{_raw} = $self->_get($req);
 	use WWW::CNic::Response::CancelTransfer;
-	return WWW::CNic::Response::CancelTransfer->new($self->{_response}{_raw});
+	return WWW::CNic::Response::CancelTransfer->new($self->{_response}->{_raw});
+}
+
+sub _reactivate_domain {
+	my $self = shift;
+	die("Missing username") if $self->{_username} eq '';
+	die("Missing password") if $self->{_password} eq '';
+	die("Missing domain")	if $self->{_domain}   eq '';
+	my %params = (
+				user		=> $self->{_username},
+				password	=> $self->_crypt_md5($self->{_password}),
+				test		=> $self->{_test},
+				domain		=> $self->{_domain},
+	);
+	$params{email} = $self->{_email} if (defined($self->{email}));
+	my $url = "$self->{_base}/reactivation_request";
+	my $req = POST($url, \%params);
+	$self->{_response}->{_raw} = $self->_get($req);
+	use WWW::CNic::Response::ReactivateDomain;
+	return WWW::CNic::Response::ReactivateDomain->new($self->{_response}->{_raw});
 }
 
 sub _get {
@@ -606,7 +681,7 @@ sub _build_handle_string {
 	my ($self, $data) = @_;
 	my @str = ('new');
 	foreach my $key(qw(name company address postcode country phone fax email)) {
-		push(@str, ${$data}{$key});
+		push(@str, ${$data}->{$key});
 	}
 	return join("::", @str);
 }
