@@ -1,7 +1,7 @@
-# Copyright (c) 2005 CentralNic Ltd. All rights reserved. This program is
+# Copyright (c) 2006 CentralNic Ltd. All rights reserved. This program is
 # free software; you can redistribute it and/or modify it under the same
 # terms as Perl itself.
-# $Id: Cookbook.pm,v 1.23 2005/04/07 10:55:46 gavin Exp $
+# $Id: Cookbook.pm,v 1.33 2006/01/09 10:46:22 gavin Exp $
 
 package WWW::CNic::Cookbook;
 use vars qw($VERSION);
@@ -158,6 +158,35 @@ These values can be accessed in the following way:
 
 The C<dns> key is an array. Elements in the array may be either a plain scalar containing the DNS hostname of the server, or an anonymous array containing the DNS hostname and IPV4 address in that order. Use the C<ref()> function to work out which.
 
+=head2 Getting Detailed Information About a Handle
+
+This command can be used to retrieve information about a handle I<for which you are the sponsor>. You are the sponsor of a handle if you created it as part of a domain registration or modification process, or it is associated with a domain name for which you are Billing Handle.
+
+	use WWW::CNic;
+
+	my $query = WWW::CNic->new(
+		command		=> 'handle_info',
+		use_ssl		=> 0,
+		username	=> $handle,
+		password	=> $passwd,
+	);
+
+	$query->set(handle => 'H12345');
+
+	my $response = $query->execute;
+
+	if ($response->is_error) {
+		printf("Error: %s\n", $response->error);
+
+	} else {
+		foreach my $key (qw(name company address postcode country tel fax email visible)) {
+			printf("%s: %s\n", $key, $response->response($key));
+		}
+
+	}
+
+The C<visible> value is a binary value (1 or 0) that indicates whether this handle's details are show in WHOIS records for domains with which it is associated.
+
 =head1 Creating New Domains and Handles
 
 =head2 Creating a Handle
@@ -186,6 +215,9 @@ To do so, use the C<create_handle> command:
 		email	=> 'jd@example.com',
 	});
 
+	# make the handle hidden on whois records:
+	$query->set(visible => 0);
+
 	my $response = $query->execute;
 
 	if ($response->is_success) {
@@ -215,6 +247,8 @@ where C<AA> is the dialling code for the country in question, C<BBB> is the phon
 
 The C<country> field must be a valid ISO3166 2-character country code.
 
+The C<visible> parameter controls whether the handle's contact details are shown on whois records for domain names to which it is associated. By default, the value of C<visible> is C<1>. A value of C<0> hides the handle from whois records.
+
 You can use the C<handle()> method to return the ID of the created handle, for use when registering.
 
 =head2 Registering a Domain Name
@@ -292,7 +326,7 @@ C<period> - the registration period for the domain name. This is an integer numb
 
 The response object for this command has all the usual methods (as documented in L<WWW::CNic::Response>), plus the C<amount()> method, which returns the price in Sterling for the domain.
 
-=head1 Modifying Domain Names
+=head1 Modifying Domain Names and Handles
 
 =head2 Modifying a Domain Name
 
@@ -311,8 +345,8 @@ You can use C<WWW::CNic> to do real-time modification of a domain. The procedure
 	$query->set(
 		thandle	=> $handle,
 		dns	=> {
-			drop	=> ['ns0.centralnic.net', 'ns1.centralnic.net'],
-			add	=> ['ns0.example.com,',   'ns1.example.com'],
+			drop	=> 'all',
+			add	=> ['ns0.example.com',   'ns1.example.com'],
 		}
 	);
 
@@ -336,7 +370,7 @@ C<thandle> corresponds to a new Technical Handle. The allowed values are the sam
 
 =item 2
 
-C<dns> must be an anonymous hash (or a reference to a hash) with two keys: C<add> and C<drop>. Their values are anonymous arrays of DNS hostnames.
+C<dns> must be an anonymous hash (or a reference to a hash) with two keys: C<add> and C<drop>. Their values are anonymous arrays of DNS hostnames. When dropping DNS servers, you can use the string C<'all'> to indicate that you want to delete all the previously delegated DNS servers (this doesn't affect any servers you might add during the same transaction).
 
 =back
 
@@ -364,6 +398,58 @@ If you create a new Technical Handle, you can use the C<response()> method of th
 
 	}
 
+=head2 Modifying a Handle
+
+When a handle is created by the C<create_handle> or C<register> commands, or via other interfaces such as the Reseller Console, it is associated with your account. This allows you to update the contact details for that handle.
+
+B<IMPORTANT>: this command B<must not> be used to completely change the contact details for a handle for a domain name. Instead you should use the C<modify> function to change the ID of the handle on the domain name (see above). This function is only for incremental changes to contact details.
+
+	use WWW::CNic;
+
+	my $query = WWW::CNic->new(
+		use_ssl		=> 1,
+		command		=> 'modify_handle',
+		username	=> $handle,
+		password	=> $passwd,
+	);
+
+	$query->set(
+		handle		=> 'H12345',
+		name		=> 'John Doe',
+		company		=> 'Example, Inc',
+		address		=> "Example House, Example Street, London",
+		postcode	=> 'EC1 123',
+		country		=> 'UK',
+		phone		=> '+44.8700170900',
+		fax		=> '+44.8700170901',
+		email		=> 'jd@example.com',
+		visible		=> 0,
+	);
+
+	my $response = $query->execute;
+
+	if ($response->is_success) {
+		print "Handle modified OK.\n";
+
+	} else {
+		printf("Error, could not modify handle: %s\n", $response->error);
+
+	}
+
+You can change the following values:
+
+	name
+	company
+	address
+	postcode
+	country
+	phone
+	fax
+	email
+	visible
+
+The meaning of these fields is identical to those described in the "Creating a Handle" section. Note that C<name>, C<country> and C<email> are all mandatory and cannot be blank.
+
 =head2 Renewing a Domain Name
 
 	use WWW::CNic;
@@ -388,7 +474,7 @@ If you create a new Technical Handle, you can use the C<response()> method of th
 
 	}
 
-You can issue advance renewals for domains using this command. You simply set a C<domains> parameter to be an anonymous array of domain names.
+You can issue advance renewals for domains using this command. You simply set a C<domains> parameter to be an anonymous array of domain names, or a reference to an array (eg C<\@domains>).
 
 Under normal circumstances, a renewal invoice or proforma is not issued right away, but is queued until the end of the day. If you want to generate a renewal invoice immediately, set the C<immediate> parameter, like so:
 
@@ -482,6 +568,37 @@ I<Declining renewal> means that a domain name will not be renewed when it expire
 
 This command lets you reinstate a domain name previously placed on Declined Renewal. When the domain name reaches its expiry, a renewal will be issued as normal.
 
+=head2 Requesting Reactivation of a Domain Name
+
+Domains that are on the "Pending Deletion" status may be reactivated upon
+request. This function provides a way to automatically submit a reactivation
+request. When we receive your request, it will be processed by a member of our
+Domain Administration team. The domain will then be placed back on the "Live"
+status, and a registration or renewal invoice will be re-issed.
+
+	my $query = WWW::CNic->new(
+		use_ssl		=> 1,
+		command		=> 'reactivate',
+		domain		=> $domain,
+		username	=> $handle,
+		password	=> $password,
+	);
+
+	$query->set('email', 'yourname@example.com');
+
+	my $response = $query->execute;
+
+	if ($response->is_success) {
+		print "Reactivation request accepted.\n";
+
+	} else {
+		printf("Error: %s\n", $response->error);
+
+	}
+	
+If you specify an C<email> parameter, our administrators will send a notification
+to that address.
+
 =head1 Transferring Domain Names
 
 =head2 Starting a Domain Transfer
@@ -496,7 +613,7 @@ You can B<start> a transfer process for one or more domain names using the C<sta
 
 4. If the transfer was B<rejected>, the B<gaining> registrar is notified.
 
-5. If the transfer was B<approved>, the object is transferred to the B<gaining> registrar. Any outstanding payments for the domain name are also transferred to the B<gaining> registrar.
+5. If the transfer was B<approved>, the object is transferred to the B<gaining> registrar. Any outstanding payments for the domain name are also transferred to the B<gaining> registrar. If the Technical Contact for the domain name matches the B<losing> registrar, then this is also changed to the B<gaining> registrar.
 
 6. If the transfer is not explicitly I<approved> or I<rejected> by the B<losing> registrar B<within five calendar days>, then the transfer is automatically marked as C<rejected>. The transfer may still be actioned if the B<gaining> registrar can acquire written authorisation (ideally on company letterhead) from the Registrant.
 
@@ -669,7 +786,7 @@ We're always keen to find out about how people are using our Toolkit system, and
 
 =head1 Copyright
 
-This module is (c) 2005 CentralNic Ltd. All rights reserved. This module is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
+This module is (c) 2006 CentralNic Ltd. All rights reserved. This module is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
 
 =head1 See Also
 
